@@ -43,7 +43,7 @@ from ._chemicals import Chemicals, CompiledChemicals
 from ._thermal_condition import ThermalCondition
 from . import mixture
 from .mixture import Mixture
-from ._thermo import Thermo
+from ._thermo import Thermo, IdealThermo
 from ._settings import settings
 from ._thermo_data import ThermoData
 from . import (indexer,
@@ -55,11 +55,12 @@ from ._multi_stream import MultiStream
 from .base import functor
 from .reaction import *
 
-__all__ = ('Chemical', 'Chemicals', 'CompiledChemicals', 'Mixture', 'Thermo', 'Stream',
-           'MultiStream', 'ThermalCondition', 'mixture', 'ThermoData',
-           *reaction.__all__, 'indexer', 'settings', 'functor', 'functors', 
-           'chemicals', 'base', 'equilibrium', 'units_of_measure', 'exceptions',
-           'functional', 'reaction', 'constants', 'utils', 'separations')
+__all__ = ('Chemical', 'Chemicals', 'CompiledChemicals', 'Mixture', 'Thermo', 
+           'IdealThermo', 'Stream', 'MultiStream', 'ThermalCondition', 
+           'mixture', 'ThermoData', *reaction.__all__, 'indexer', 'settings', 
+           'functor', 'functors', 'chemicals', 'base', 'equilibrium', 
+           'units_of_measure', 'exceptions', 'functional', 'reaction', 
+           'constants', 'utils', 'separations')
 
 def speed_up():
     from warnings import warn
@@ -80,4 +81,4 @@ pd.set_option('display.max_columns', 10)
 pd.set_option('max_colwidth', 35)
 del np, pd
 
-__version__ = "0.27.3"
+__version__ = "0.27.8"
