@@ -191,21 +191,45 @@ class MultiStream(Stream):
                  thermo=None, **phase_flows):
         self._thermal_condition = ThermalCondition(T, P)
         thermo = self._load_thermo(thermo)
-        self._init_indexer(flow, phases, thermo.chemicals, phase_flows)
+        chemicals = thermo.chemicals
         self.price = price
         if units:
             name, factor = self._get_flow_name_and_factor(units)
+            if name == 'mass':
+                group_wt_compositions = chemicals._group_wt_compositions
+                for phase, chemical_flows in phase_flows.items():
+                    new_chemical_flows = []
+                    for item in chemical_flows:
+                        cID, value = item
+                        if cID in group_wt_compositions:
+                            compositions = group_wt_compositions[cID]
+                            chemical_group = chemicals[cID]
+                            for i in range(len(chemical_group)):
+                                item = (chemical_group[i]._ID, value * compositions[i])
+                                new_chemical_flows.append(item)
+                        else:
+                            new_chemical_flows.append(item)
+                    phase_flows[phase] = new_chemical_flows
+            elif name == 'vol':
+                group_wt_compositions = chemicals._group_wt_compositions
+                for chemical_flows in phase_flows.values():
+                    for item in chemical_flows:
+                        cID, value = item
+                        if cID in group_wt_compositions:
+                            raise ValueError(f"cannot set volumetric flow by chemical group '{i}'")
+            self._init_indexer(flow, phases, chemicals, phase_flows)
             flow = getattr(self, 'i' + name)
             material_data = self._imol._data * factor
             if total_flow: material_data *= total_flow / material_data.sum()
             flow._data[:] = material_data
-        elif total_flow:
-            self._imol._data *= total_flow / self.F_mol
+        else:
+            self._init_indexer(flow, phases, chemicals, phase_flows)
+            if total_flow: self._imol._data *= total_flow / self.F_mol
         self._sink = self._source = None
         self.reset_cache()
         self._register(ID)
         self._link = None
-            
+        
     def _init_indexer(self, flow, phases, chemicals, phase_flows):
         if flow == ():
             if phase_flows:
@@ -220,7 +244,21 @@ class MultiStream(Stream):
             else:
                 imol = MolarFlowIndexer.from_data(flow, phases, chemicals)
         self._imol = imol
-        
+    
+    def _get_property_cache(self, name):
+        property_cache = self._property_cache
+        thermal_condition = self._thermal_condition
+        imol = self._imol
+        data = imol._data
+        literal = (imol._phases, thermal_condition._T, thermal_condition._P)
+        last_literal, last_data = self._property_cache_key
+        if literal == last_literal and (data == last_data).all():
+            return property_cache.get(name) 
+        else:
+            self._property_cache_key = (literal, data.copy())
+            property_cache.clear()
+            return None
+    
     def reset_cache(self):
         """Reset cache regarding equilibrium methods."""
         super().reset_cache()
@@ -252,6 +290,8 @@ class MultiStream(Stream):
             stream._thermo = self._thermo
             stream._bubble_point_cache =  self._bubble_point_cache
             stream._dew_point_cache = self._dew_point_cache
+            stream._property_cache = {}
+            stream._property_cache_key = None, None
             streams[phase] = stream
         return stream
     
@@ -396,7 +436,10 @@ class MultiStream(Stream):
     @property
     def H(self):
         """[float] Enthalpy flow rate in kJ/hr."""
-        return self.mixture.xH(self._imol, *self._thermal_condition)
+        H = self._get_property_cache('H')
+        if H is None:
+            self._property_cache['H'] = H = self.mixture.xH(self._imol, *self._thermal_condition)
+        return H
     @H.setter
     def H(self, H):
         self.T = self.mixture.xsolve_T(self._imol, H, *self._thermal_condition)
@@ -404,15 +447,21 @@ class MultiStream(Stream):
     @property
     def S(self):
         """[float] Absolute entropy flow rate in kJ/hr."""
-        return self.mixture.xS(self._imol, *self._thermal_condition)
+        S = self._get_property_cache('S')
+        if S is None:
+            self._property_cache['S'] = S = self.mixture.xS(self._imol, *self._thermal_condition)
+        return S
     @property
     def C(self):
         """[float] Heat capacity flow rate in kJ/hr."""
-        return self.mixture.xCn(self._imol, self._thermal_condition.T)
+        C = self._get_property_cache('C')
+        if C is None:
+            self._property_cache['C'] = C = self.mixture.xCn(self._imol, self.T)
+        return C
     @property
     def F_vol(self):
         """[float] Total volumetric flow rate in m3/hr."""
-        return 1000. * self.mixture.xV(self._imol, *self._thermal_condition)
+        return 1000. * self.V * self.F_mol
     @F_vol.setter
     def F_vol(self, value):
         F_vol = self.F_vol
@@ -422,7 +471,15 @@ class MultiStream(Stream):
     @property
     def Hvap(self):
         """[float] Enthalpy of vaporization flow rate in kJ/hr."""
-        return self.mixture.Hvap(self._imol['l'], *self._thermal_condition)
+        mol = self._imol['l']
+        T = self._thermal_condition._T
+        Hvap = self._get_property_cache('Hvap')
+        if Hvap is None:
+            self._property_cache['Hvap'] = Hvap = sum([
+                i*j.Hvap(T) for i,j in zip(mol, self.chemicals)
+                if i and not j.locked_state
+            ])
+        return Hvap
     
     ### Composition properties ###
     
@@ -442,49 +499,47 @@ class MultiStream(Stream):
     @property
     def V(self):
         """[float] Molar volume [m^3/mol]."""
-        return self.mixture.xV(self._imol.iter_composition(), *self._thermal_condition)
+        V = self._get_property_cache('V')
+        if V is None:
+            self._property_cache['V'] = V = self.mixture.xV(self._imol.iter_composition(), *self._thermal_condition)
+        return V
     @property
     def kappa(self):
         """[float] Thermal conductivity [W/m/k]."""
-        return self.mixture.xkappa(self._imol.iter_composition(), *self._thermal_condition)        
+        kappa = self._get_property_cache('kappa')
+        if kappa is None:
+            self._property_cache['kappa'] = kappa = self.mixture.xkappa(self._imol.iter_composition(), *self._thermal_condition)
+        return kappa      
     @property
     def Cn(self):
         """[float] Molar heat capacity [J/mol/K]."""
-        return self.mixture.xCn(self._imol.iter_composition(), self.T)
+        Cn = self._get_property_cache('Cn')
+        if Cn is None:
+            self._property_cache['Cn'] = Cn = self.mixture.xCn(self._imol.iter_composition(), self.T)
+        return Cn
     @property
     def mu(self):
         """[float] Hydrolic viscosity [Pa*s]."""
-        return self.mixture.xmu(self._imol.iter_composition(), *self._thermal_condition)
+        mu = self._get_property_cache('mu')
+        if mu is None:
+            self._property_cache['mu'] = mu = self.mixture.xmu(self._imol.iter_composition(), *self._thermal_condition)
+        return mu
     @property
     def sigma(self):
         """[float] Surface tension [N/m]."""
         mol = self._imol['l']
-        return self.mixture.sigma(mol / mol.sum(), *self._thermal_condition)
+        sigma = self._get_property_cache('sigma')
+        if sigma is None:
+            self._property_cache['sigma'] = sigma = self.mixture.sigma(mol / mol.sum(), *self._thermal_condition)
+        return sigma
     @property
     def epsilon(self):
         """[float] Relative permittivity [-]."""
         mol = self._imol['l']
-        return self.mixture.epsilon(mol / mol.sum(), *self._thermal_condition)
-    @property
-    def Cp(self):
-        """[float] Heat capacity [J/g/K]."""
-        return self.mixture.xCp(self._imol.iter_composition(), self.T)
-    @property
-    def alpha(self):
-        """[float] Thermal diffusivity [m^2/s]."""
-        return self.mixture.xalpha(self._imol.iter_composition(), *self._thermal_condition)
-    @property
-    def rho(self):
-        """[float] Density [kg/m^3]."""
-        return self.mixture.xrho(self._imol.iter_composition(), *self._thermal_condition)
-    @property
-    def nu(self):
-        """[float] Kinematic viscosity [m^2/s]."""
-        return self.mixture.xnu(self._imol.iter_composition(), *self._thermal_condition)
-    @property
-    def Pr(self):
-        """[float] Prandtl number [-]."""
-        return self.mixture.xPr(self._imol.iter_composition(), *self._thermal_condition)
+        epsilon = self._get_property_cache('epsilon')
+        if epsilon is None:
+            self._property_cache['epsilon'] = epsilon = self.mixture.epsilon(mol / mol.sum(), *self._thermal_condition)
+        return epsilon
         
     ### Methods ###
     
