@@ -9,6 +9,7 @@
 """
 import numpy as np
 import thermosteam as tmo
+import flexsolve as flx
 from thermosteam import functional as fn
 from . import indexer
 from . import equilibrium as eq
@@ -236,7 +237,8 @@ class Stream:
     __slots__ = ('_ID', '_imol', '_thermal_condition', '_thermo', '_streams',
                  '_bubble_point_cache', '_dew_point_cache',
                  '_vle_cache', '_lle_cache', '_sle_cache',
-                 '_sink', '_source', '_price', '_link')
+                 '_sink', '_source', '_price', '_link',
+                 '_property_cache_key', '_property_cache')
     line = 'Stream'
     
     #: [DisplayUnits] Units of measure for IPython display (class attribute)
@@ -252,22 +254,50 @@ class Stream:
                  **chemical_flows):
         self._thermal_condition = tmo.ThermalCondition(T, P)
         thermo = self._load_thermo(thermo)
-        self._init_indexer(flow, phase, thermo.chemicals, chemical_flows)
+        chemicals = thermo.chemicals
         self.price = price
-        mol = self.mol
         if units:
             name, factor = self._get_flow_name_and_factor(units)
+            if name == 'mass':
+                group_wt_compositions = chemicals._group_wt_compositions
+                for cID in tuple(chemical_flows):
+                    if cID in group_wt_compositions:
+                        compositions = group_wt_compositions[cID]
+                        group_flow = chemical_flows.pop(cID)
+                        chemical_group = chemicals[cID]
+                        for i in range(len(chemical_group)):
+                            chemical_flows[chemical_group[i]._ID] = group_flow * compositions[i]
+            elif name == 'vol':
+                group_wt_compositions = chemicals._group_wt_compositions
+                for cID in chemical_flows:
+                    if cID in group_wt_compositions:
+                        raise ValueError(f"cannot set volumetric flow by chemical group '{i}'")
+            self._init_indexer(flow, phase, chemicals, chemical_flows)
+            mol = self.mol
             flow = getattr(self, name)
+            if total_flow is not None: mol *= total_flow / mol.sum()
             material_data = mol / factor
-            if total_flow: material_data *= total_flow / material_data.sum()
             flow[:] = material_data
-        elif total_flow:
-            mol *= total_flow / mol.sum()
+        else:
+            self._init_indexer(flow, phase, chemicals, chemical_flows)
+            if total_flow:
+                mol = self.mol
+                mol *= total_flow / mol.sum()
         self._sink = self._source = None # For BioSTEAM
         self.reset_cache()
         self._register(ID)
         self._link = None
 
+    def _reset_thermo(self, thermo):
+        self._thermo = thermo
+        self._imol.reset_chemicals(thermo.chemicals)
+        self._link = None
+        self.reset_cache()
+        if hasattr(self, '_streams'):
+            for phase, stream in self._streams.items():
+                stream._imol = self._imol.get_phase(phase)
+                stream._thermo = thermo
+
     def shares_flow_rate_with(self, other):
         """
         Return whether other stream shares data with this one.
@@ -363,8 +393,6 @@ class Stream:
             self.phases = stream_data._phases
             self._imol.copy_like(stream_data._imol)
             self._thermal_condition.copy_like(stream_data)
-        elif stream_data is None:
-            self.empty()
         else:
             raise ValueError(f'stream_data must be a StreamData object; not {type(stream_data).__name__}')
         
@@ -485,6 +513,8 @@ class Stream:
         """Reset cache regarding equilibrium methods."""
         self._bubble_point_cache = eq.BubblePointCache()
         self._dew_point_cache = eq.DewPointCache()
+        self._property_cache_key = None, None
+        self._property_cache = {}
 
     @classmethod
     def _get_flow_name_and_factor(cls, units):
@@ -719,26 +749,26 @@ class Stream:
     @property
     def T(self):
         """[float] Temperature in Kelvin."""
-        return self._thermal_condition.T
+        return self._thermal_condition._T
     @T.setter
     def T(self, T):
-        self._thermal_condition.T = T
+        self._thermal_condition._T = float(T)
     
     @property
     def P(self):
         """[float] Pressure in Pascal."""
-        return self._thermal_condition.P
+        return self._thermal_condition._P
     @P.setter
     def P(self, P):
-        self._thermal_condition.P = P
+        self._thermal_condition._P = float(P)
     
     @property
     def phase(self):
         """Phase of stream."""
-        return self._imol.phase
+        return self._imol._phase._phase
     @phase.setter
     def phase(self, phase):
-        self._imol.phase = phase
+        self._imol._phase.phase = phase
     
     @property
     def mol(self):
@@ -747,8 +777,7 @@ class Stream:
     @mol.setter
     def mol(self, value):
         mol = self.mol
-        if mol is not value:
-            mol[:] = value
+        if mol is not value: mol[:] = value
     
     @property
     def mass(self):
@@ -757,8 +786,7 @@ class Stream:
     @mass.setter
     def mass(self, value):
         mass = self.mass
-        if mass is not value:
-            mass[:] = value
+        if mass is not value: mass[:] = value
     
     @property
     def vol(self):
@@ -811,7 +839,8 @@ class Stream:
     @property
     def F_vol(self):
         """[float] Total volumetric flow rate in m3/hr."""
-        return 1000. * self.mixture.V(self.phase, self.mol, *self._thermal_condition)
+        F_mol = self.F_mol
+        return 1000. * self.V * F_mol if F_mol else 0.
     @F_vol.setter
     def F_vol(self, value):
         F_vol = self.F_vol
@@ -821,7 +850,12 @@ class Stream:
     @property
     def H(self):
         """[float] Enthalpy flow rate in kJ/hr."""
-        return self.mixture.H(self.phase, self.mol, *self._thermal_condition)
+        H = self._get_property_cache('H')
+        if H is None:
+            self._property_cache['H'] = H = self.mixture.H(
+                self.phase, self.mol, *self._thermal_condition
+            )
+        return H
     @H.setter
     def H(self, H: float):
         if not H and self.isempty(): return
@@ -866,13 +900,36 @@ class Stream:
     def Hvap(self):
         """[float] Enthalpy of vaporization flow rate in kJ/hr."""
         mol = self.mol
-        condition = self._thermal_condition
-        return sum([i*j.Hvap(condition.T) for i,j in zip(mol, self.chemicals) if i and not j.locked_state])
+        T = self._thermal_condition._T
+        Hvap = self._get_property_cache('Hvap')
+        if Hvap is None:
+            self._property_cache['Hvap'] = Hvap = sum([
+                i*j.Hvap(T) for i,j in zip(mol, self.chemicals)
+                if i and not j.locked_state
+            ])
+        return Hvap
+    
+    def _get_property_cache(self, name):
+        property_cache = self._property_cache
+        thermal_condition = self._thermal_condition
+        imol = self._imol
+        data = imol._data
+        literal = (imol._phase._phase, thermal_condition._T, thermal_condition._P)
+        last_literal, last_data = self._property_cache_key
+        if literal == last_literal and (data == last_data).all():
+            return property_cache.get(name) 
+        else:
+            self._property_cache_key = (literal, data.copy())
+            property_cache.clear()
+            return None
     
     @property
     def C(self):
         """[float] Heat capacity flow rate in kJ/hr."""
-        return self.mixture.Cn(self.phase, self.mol, self.T)
+        C = self._get_property_cache('C')
+        if C is None:
+            self._property_cache['C'] = C = self.mixture.Cn(self.phase, self.mol, self.T)
+        return C
     
     ### Composition properties ###
     
@@ -905,57 +962,88 @@ class Stream:
     @property
     def V(self):
         """[float] Molar volume [m^3/mol]."""
-        return self.mixture.V(*self._imol.get_phase_and_composition(),
-                              *self._thermal_condition)
+        V = self._get_property_cache('V')
+        if V is None:
+            self._property_cache['V'] = V = self.mixture.V(
+                *self._imol.get_phase_and_composition(),
+                *self._thermal_condition
+            )
+        return V
+        
     @property
     def kappa(self):
         """[float] Thermal conductivity [W/m/k]."""
-        return self.mixture.kappa(*self._imol.get_phase_and_composition(),
-                                  *self._thermal_condition)
+        kappa = self._get_property_cache('kappa')
+        if kappa is None:
+            self._property_cache['kappa'] = kappa = self.mixture.kappa(
+                *self._imol.get_phase_and_composition(),
+                *self._thermal_condition
+            )
+        return kappa
     @property
     def Cn(self):
         """[float] Molar heat capacity [J/mol/K]."""
-        return self.mixture.Cn(*self._imol.get_phase_and_composition(), self.T)
+        Cn = self._get_property_cache('Cn')
+        if Cn is None:
+            self._property_cache['Cn'] = Cn = self.mixture.Cn(
+                *self._imol.get_phase_and_composition(),
+                self.T
+            )
+        return Cn
     @property
     def mu(self):
         """[float] Hydrolic viscosity [Pa*s]."""
-        return self.mixture.mu(*self._imol.get_phase_and_composition(),
-                               *self._thermal_condition)
+        mu = self._get_property_cache('mu')
+        if mu is None:
+            self._property_cache['mu'] = mu = self.mixture.mu(
+                *self._imol.get_phase_and_composition(),
+                *self._thermal_condition
+            )
+        return mu
     @property
     def sigma(self):
         """[float] Surface tension [N/m]."""
         mol = self.mol
-        return self.mixture.sigma(mol / mol.sum(),
-                                  *self._thermal_condition)
+        sigma = self._get_property_cache('sigma')
+        if sigma is None:
+            self._property_cache['sigma'] = sigma = self.mixture.sigma(
+                mol / mol.sum(), *self._thermal_condition
+            )
+        return sigma
     @property
     def epsilon(self):
         """[float] Relative permittivity [-]."""
         mol = self.mol
-        return self.mixture.epsilon(mol / mol.sum(), *self._thermal_condition)
+        epsilon = self._get_property_cache('epsilon')
+        if epsilon is None:
+            self._property_cache['epsilon'] = epsilon = self.mixture.epsilon(
+                mol / mol.sum(), *self._thermal_condition
+            )
+        return epsilon
     @property
     def Cp(self):
         """[float] Heat capacity [J/g/K]."""
-        return self.mixture.Cp(*self._imol.get_phase_and_composition(), self.T)
+        return self.Cn / self.MW
     @property
     def alpha(self):
         """[float] Thermal diffusivity [m^2/s]."""
-        return self.mixture.alpha(*self._imol.get_phase_and_composition(),
-                                  *self._thermal_condition)
+        return fn.alpha(self.kappa, 
+                        self.rho, 
+                        self.Cp * 1000.)
     @property
     def rho(self):
         """[float] Density [kg/m^3]."""
-        return self.mixture.rho(*self._imol.get_phase_and_composition(), 
-                                *self._thermal_condition)
+        return fn.V_to_rho(self.V, self.MW)
     @property
     def nu(self):
         """[float] Kinematic viscosity [m^2/s]."""
-        return self.mixture.nu(*self._imol.get_phase_and_composition(),
-                               *self._thermal_condition)
+        return fn.mu_to_nu(self.mu, self.rho)
     @property
     def Pr(self):
         """[float] Prandtl number [-]."""
-        return self.mixture.Pr(*self._imol.get_phase_and_composition(),
-                               *self._thermal_condition)
+        return fn.Pr(self.Cp * 1000,
+                     self.kappa, 
+                     self.mu)
     
     ### Stream methods ###
     
@@ -982,7 +1070,7 @@ class Stream:
         return self._thermal_condition.in_equilibrium(other._thermal_condition)
     
     @classmethod
-    def sum(cls, streams, ID=None, thermo=None):
+    def sum(cls, streams, ID=None, thermo=None, energy_balance=True):
         """
         Return a new Stream object that represents the sum of all given streams.
         
@@ -1012,7 +1100,7 @@ class Stream:
         """
         new = cls(ID, thermo=thermo)
         if streams: new.copy_thermal_condition(streams[0])
-        new.mix_from(streams)
+        new.mix_from(streams, energy_balance)
         return new
     
     def separate_out(self, other, energy_balance=True):
@@ -1063,7 +1151,6 @@ class Stream:
             if self is other: self.empty()
             if energy_balance: H_new = self.H - other.H
             self._imol.separate_out(other._imol)
-            self.sanity_check()
             if energy_balance: self.H = H_new
     
     def mix_from(self, others, energy_balance=True):
@@ -1118,9 +1205,9 @@ class Stream:
         elif N_others == 1:
             self.copy_like(others[0])
         else:
+            if energy_balance: H = sum([i.H for i in others])
             self._imol.mix_from([i._imol for i in others])
             if energy_balance and not self.isempty():
-                H = sum([i.H for i in others])
                 try:
                     self.H = H
                 except:
@@ -1129,7 +1216,7 @@ class Stream:
                     self._imol.mix_from([i._imol for i in others])
                     self.H = H
                 
-    def split_to(self, s1, s2, split):
+    def split_to(self, s1, s2, split, energy_balance=True):
         """
         Split molar flow rate from this stream to two others given
         the split fraction or an array of split fractions.
@@ -1158,13 +1245,31 @@ class Stream:
         
         
         """
-        if self is s1: 
-            mol = self.mol.copy()
+        mol = self.mol
+        chemicals = self.chemicals
+        values = mol * split
+        dummy = mol - values
+        if s1.chemicals is chemicals: 
+            s1.mol[:] = values
         else:
-            mol = self.mol
-        
-        s1.mol[:] = dummy = mol * split
-        s2.mol[:] = mol - dummy
+            CASs, values = zip(*[(i, j) for i, j in zip(chemicals.CASs, values) if j])
+            s1.empty()
+            s1._imol[CASs] = values
+        values = dummy
+        if s2.chemicals is chemicals:
+            s2.mol[:] = values
+        else:
+            s2.empty()
+            CASs, values = zip(*[(i, j) for i, j in zip(chemicals.CASs, values) if j])
+            s2._imol[CASs] = values
+        if energy_balance:
+            tc1 = s1._thermal_condition
+            tc2 = s2._thermal_condition
+            tc = self._thermal_condition
+            tc1._T = tc2._T = tc._T
+            tc1._P = tc2._P = tc._P
+            s1.phase = s2.phase = self.phase
+            
         
     def link_with(self, other, flow=True, phase=True, TP=True):
         """
@@ -1314,13 +1419,13 @@ class Stream:
         """
         self._thermal_condition.copy_like(other._thermal_condition)
     
-    def copy_flow(self, stream, IDs=..., *, remove=False, exclude=False):
+    def copy_flow(self, other, IDs=..., *, remove=False, exclude=False):
         """
-        Copy flow rates of stream to self.
+        Copy flow rates of another stream to self.
         
         Parameters
         ----------
-        stream : Stream
+        other : Stream
             Flow rates will be copied from here.
         IDs=... : Iterable[str], defaults to all chemicals.
             Chemical IDs. 
@@ -1393,24 +1498,58 @@ class Stream:
          phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
          flow: 0
          
+        Copy flows except except water and remove water:
+        >>> s1 = tmo.Stream('s1', Water=20, Ethanol=10, units='kg/hr')
+        >>> s2 = tmo.Stream('s2')
+        >>> s2.copy_flow(s1, 'Water', exclude=True, remove=True)
+         
         """
+        other_mol = other.mol
+        other_chemicals = other.chemicals
         chemicals = self.chemicals
-        mol = stream.mol
-        if exclude:
-            IDs = chemicals.get_index(IDs)
-            index = np.ones(chemicals.size, dtype=bool)
-            index[IDs] = False
-        else:
-            index = chemicals.get_index(IDs)
-        
-        self.mol[index] = mol[index]
-        if remove: 
-            if isinstance(stream, tmo.MultiStream):
-                stream.imol.data[:, index] = 0
+        if IDs == ...:
+            if exclude: return 
+            if chemicals is other_chemicals:
+                self.mol[:] = other.mol
             else:
-                mol[index] = 0
+                self.empty()
+                CASs, values = zip(*[(i, j) for i, j in zip(other_chemicals.CASs, other_mol) if j])
+                self.imol[CASs] = values
+            if remove: 
+                if isinstance(other, tmo.MultiStream):
+                    other.imol.data[:] = 0.
+                else:
+                    other_mol[:] = 0.
+        else:
+            if exclude:
+                if isinstance(IDs, str):
+                    if IDs in other_chemicals:
+                        bad_index = other_chemicals.index(IDs)
+                        other_index = [i for i in range(other_chemicals.size) if i != bad_index]
+                    else:
+                        other_index = slice()
+                else:
+                    IDs = [i for i in IDs if i in other_chemicals]
+                    bad_index = set(other_chemicals.indices(IDs))
+                    if bad_index:
+                        other_index = [i for i in range(other_chemicals.size) if i not in bad_index]
+                    else:
+                        other_index = slice()
+            else:
+                other_index = other_chemicals.get_index(IDs)
+            if chemicals is other_chemicals:
+                self.mol[other_index] = other_mol[other_index]
+            else:
+                CASs = other_chemicals.CASs
+                other_index = [i for i in other_index if other_mol[i] or CASs[i] in chemicals]
+                self.imol[tuple([CASs[i] for i in other_index])] = other_mol[other_index]
+            if remove: 
+                if isinstance(other, tmo.MultiStream):
+                    other.imol.data[:, other_index] = 0
+                else:
+                    other_mol[other_index] = 0
     
-    def copy(self, ID=None):
+    def copy(self, ID=None, thermo=None):
         """
         Return a copy of the stream.
 
@@ -1436,8 +1575,10 @@ class Stream:
         cls = self.__class__
         new = cls.__new__(cls)
         new._link = new._sink = new._source = None
-        new._thermo = self._thermo
+        new._thermo = thermo or self._thermo
         new._imol = self._imol.copy()
+        if thermo and thermo.chemicals is not self.chemicals:
+            new._imol.reset_chemicals(thermo.chemicals)
         new._thermal_condition = self._thermal_condition.copy()
         new.reset_cache()
         new.price = 0
@@ -1493,6 +1634,10 @@ class Stream:
         new._thermo = self._thermo
         new._imol = self._imol
         new._thermal_condition = self._thermal_condition
+        new._property_cache = self._property_cache
+        new._property_cache_key = self._property_cache_key
+        new._bubble_point_cache = self._bubble_point_cache
+        new._dew_point_cache = self._dew_point_cache
         try: new._vle_cache = self._vle_cache
         except AttributeError: pass
         new._link = self
@@ -1511,7 +1656,7 @@ class Stream:
         0.0
         
         """
-        self._imol.empty()
+        self._imol._data[:] = 0.
     
     ### Equilibrium ###
     
@@ -1852,55 +1997,88 @@ class Stream:
         if x.sum() < 1e-12: return 0
         return F_l(x, self.T).sum()
     
-    def receive_vent(self, other, accumulate=False, fraction=1.0):
+    def receive_vent(self, other, energy_balance=True):
         """
-        Receive vapors from another stream as if in equilibrium.
+        Receive vapors from another stream by vapor-liquid equilibrium between 
+        a gas and liquid stream assuming only a small amount of chemicals
+        in vapor-liquid equilibrium is present
 
         Examples
         --------
+        The energy balance is performed by default:
+        
         >>> import thermosteam as tmo
         >>> chemicals = tmo.Chemicals(['Water', 'Ethanol', 'Methanol', tmo.Chemical('N2', phase='g')], cache=True)
         >>> tmo.settings.set_thermo(chemicals) 
-        >>> s1 = tmo.Stream('s1', N2=10, units='m3/hr', phase='g', T=330)
+        >>> s1 = tmo.Stream('s1', N2=20, units='m3/hr', phase='g', T=330)
         >>> s2 = tmo.Stream('s2', Water=10, Ethanol=2, T=330)
-        >>> s1.receive_vent(s2, accumulate=True)
+        >>> s1.receive_vent(s2)
+        >>> s1.show(flow='kmol/hr')
+        Stream: s1
+         phase: 'g', T: 323.13 K, P: 101325 Pa
+         flow (kmol/hr): Water    0.0798
+                         Ethanol  0.0889
+                         N2       0.739
+        
+        Set energy balance to false to receive vent isothermally:
+            
+        >>> import thermosteam as tmo
+        >>> chemicals = tmo.Chemicals(['Water', 'Ethanol', 'Methanol', tmo.Chemical('N2', phase='g')], cache=True)
+        >>> tmo.settings.set_thermo(chemicals) 
+        >>> s1 = tmo.Stream('s1', N2=20, units='m3/hr', phase='g', T=330)
+        >>> s2 = tmo.Stream('s2', Water=10, Ethanol=2, T=330)
+        >>> s1.receive_vent(s2, energy_balance=False)
         >>> s1.show(flow='kmol/hr')
         Stream: s1
          phase: 'g', T: 330 K, P: 101325 Pa
-         flow (kmol/hr): Water    0.0557
-                         Ethanol  0.0616
-                         N2       0.369
+         flow (kmol/hr): Water    0.111
+                         Ethanol  0.123
+                         N2       0.739
+        
         """
-        ms = tmo.MultiStream(None)
         assert self.phase == 'g', 'stream must be a gas to receive vent'
-        chemicals = other.vle_chemicals
-        light_indices = other.chemicals._light_indices
-        if accumulate:
-            self.mol[light_indices] += other.mol[light_indices]
-        else:
-            self.mol[light_indices] = other.mol[light_indices]
-        other.mol[light_indices] = 0
-        F_l = eq.LiquidFugacities(chemicals, other.thermo)
+        ms = tmo.Stream(None, T=self.T, P=self.P, thermo=self.thermo)
+        ms.mix_from([self, other], energy_balance=False)
+        if energy_balance: ms.H = H = self.H + other.H
+        ms.vle._setup()
+        chemicals = ms.vle_chemicals
+        F_l = eq.LiquidFugacities(chemicals, ms.thermo)
         IDs = tuple([i.ID for i in chemicals])
         x = other.get_molar_composition(IDs)
-        T = self.T
-        P = self.P
-        f_l = F_l(x, T)
-        y = f_l / P
-        imol = self.imol
-        mol_old = imol[IDs]
-        mol_new = fraction * self.F_mol * y
-        if accumulate:
-            imol[IDs] += mol_new            
+        T = ms.T
+        P = ms.P
+        vapor = ms['g']
+        liquid = ms['l']
+        F_mol_vapor = vapor.F_mol
+        mol_old = liquid.imol[IDs]
+        if energy_balance:
+            def equilibrium_approximation(T):
+                f_l = F_l(x, T)
+                y = f_l / P
+                mol_new = F_mol_vapor * y
+                vapor.imol[IDs] = mol_new
+                liquid.imol[IDs] = mol_old - mol_new 
+                index = liquid.mol < 0.
+                vapor.mol[index] += liquid.mol[index]
+                liquid.mol[index] = 0
+                ms.H = H 
+                return ms.T
+            flx.wegstein(equilibrium_approximation, T)
         else:
-            imol[IDs] = mol_new
-        other.imol[IDs] += mol_old - mol_new 
-        index = other.mol < 0.
-        self.mol[index] += other.mol[index]
-        other.mol[index] = 0
-        ms.mix_from([self, other])
+            f_l = F_l(x, T)
+            y = f_l / P
+            mol_new = F_mol_vapor * y
+            vapor.imol[IDs] = mol_new
+            liquid.imol[IDs] = mol_old - mol_new 
+            index = liquid.mol < 0.
+            vapor.mol[index] += liquid.mol[index]
+            liquid.mol[index] = 0
+        self.copy_like(vapor)
+        other.copy_like(liquid)
         self.T = other.T = ms.T
         
+        
+        
     ### Casting ###
     
     @property
