@@ -56,26 +56,23 @@ def set_reaction_basis(rxn, basis):
 def as_material_array(material, basis, phases, chemicals):
     isa = isinstance
     if isa(material, np.ndarray):
-        return material
+        return material, None
     elif isa(material, tmo.Stream):
         if (phases or len(material.phases) != 1) and material.phases != phases:
             raise ValueError("reaction and stream phases do not match")
-        if material.chemicals is not chemicals:
-            raise ValueError("reaction and stream chemicals do not match")
+        if material.chemicals is chemicals:
+            config = None
+        else:
+            config = material.chemicals, material.imol.reset_chemicals(chemicals)
         if basis == 'mol':
-            return material.imol.data 
+            return material.imol.data, config
         elif basis == 'wt':
-            return material.imass.data
+            return material.imass.data, config
         else:
             raise ValueError("basis must be either 'mol' or 'wt'")
     else:
         raise ValueError('reaction material must be either an array or a stream')
 
-def get_array_values(array):
-    return array.value if isinstance(array, property_array) else array
-
-def set_array_values(array, values):
-    if array is not values: array[:] = values
 
 # %%
 
@@ -272,6 +269,15 @@ class Reaction:
             self._stoichiometry = np.zeros(chemicals.size)
             self._X_index = self._chemicals.index(reactant)
     
+    @property
+    def reaction_chemicals(self):
+        """Return all chemicals involved in the reaction."""
+        return [i for i,j in zip(self._chemicals, self._stoichiometry) if j]
+    
+    def reset_chemicals(self, chemicals):
+        reaction = get_stoichiometric_string(self.stoichiometry, self.phases, self.chemicals)
+        return self.__init__(reaction, reactant=self.reactant, X=self.X, basis=self.basis, chemicals=chemicals)
+    
     def __eq__(self, other):
         try:
             return all([
@@ -370,28 +376,31 @@ class Reaction:
         return self
     
     def __call__(self, material):
-        material_array = as_material_array(material,
-                                           self._basis,
-                                           self._phases,
-                                           self._chemicals)
-        values = get_array_values(material_array)
+        material_array, config = as_material_array(
+            material, self._basis, self._phases, self._chemicals
+        )
+        isproperty = isinstance(material_array, property_array)
+        values = material_array.value if isproperty else material_array
         self._reaction(values)
         if tmo.reaction.CHECK_FEASIBILITY:
             check_material_feasibility(values)
         else:
             fn.remove_negligible_negative_values(values)
-        set_array_values(material_array, values)
+        if isproperty: material_array[:] = values
+        if config: material._imol.reset_chemicals(*config)
         
     def force_reaction(self, material):
         """React material ignoring feasibility checks."""
-        material_array = as_material_array(material,
+        material_array, config = as_material_array(material,
                                            self._basis,
                                            self._phases,
                                            self._chemicals)
-        values = get_array_values(material_array)
+        isproperty = isinstance(material_array, property_array)
+        values = material_array.value if isproperty else material_array
         self._reaction(values)
         fn.remove_negligible_negative_values(values)
-        set_array_values(material_array, values)
+        if isproperty: material_array[:] = values
+        if config: material._imol.reset_chemicals(*config)
     
     def product_yield(self, product, basis=None):
         """Return yield of product per reactant."""
@@ -848,6 +857,9 @@ class ReactionItem(Reaction):
         self._X_index = rxnset._X_index[index]
         self._index = index
     
+    def reset_chemicals(self, chemicals):
+        raise TypeError('cannot reset chemicals of a reaction item')
+    
     @property
     def basis(self):
         """{'mol', 'wt'} Basis of reaction"""
@@ -927,6 +939,19 @@ class ReactionSet:
             rxnset._chemicals = self._chemicals
             return rxnset
     
+    def reset_chemicals(self, chemicals):
+        reactions = [self[i] for i in range(self._stoichiometry.shape[0])]
+        self.__init__(
+            [Reaction(get_stoichiometric_string(i.stoichiometry, i.phases, i.chemicals), 
+                      i.reactant, i.X, chemicals, i.basis, phases=i.phases)
+             for i in reactions]
+        )
+    
+    @property
+    def reaction_chemicals(self):
+        """Return all chemicals involved in the reaction."""
+        return [i for i,j in zip(self._chemicals, self._stoichiometry.any(axis=0)) if j]
+    
     @property
     def basis(self):
         """{'mol', 'wt'} Basis of reaction"""
@@ -939,6 +964,10 @@ class ReactionSet:
     def X(self):
         """[1d array] Reaction converions."""
         return self._X
+    @X.setter
+    def X(self, X):
+        """[1d array] Reaction converions."""
+        if X is not self._X: self._X[:] = X
     
     @property
     def chemicals(self):
@@ -981,7 +1010,7 @@ class ReactionSet:
         Xs = self.X
         N = len(Xs)
         maxnumspace = max(length(string(N)) + 1, len(index_name))
-        info += f"\n{index_name}" + " "*(max(2, length(string(N)) + 1)) + "stoichiometry" + " "*(maxrxnlen - 13) + "reactant" + " "*(maxcmplen - 8) + '  X[%]'
+        info += f"\n{index_name}" + " "*(max(2, length(string(N)))) + "stoichiometry" + " "*(maxrxnlen - 13) + "reactant" + " "*(maxcmplen - 8) + '  X[%]'
         for N, rxn, cmp, X in zip(range(N), rxns, cmps, Xs):
             rxn_spaces = " "*(maxrxnlen - length(rxn))
             cmp_spaces = " "*(maxcmplen - length(cmp))
@@ -1287,6 +1316,13 @@ class ReactionSystem:
     __call__ = Reaction.__call__
     show = Reaction.show
     
+    @property
+    def X(self):
+        return [i.X for i in self._reactions]
+    @X.setter
+    def X(self, X):
+        for i, j in zip(self._reactions, X): i.X = j
+    
     @property
     def reactions(self):
         return self._reactions
@@ -1328,12 +1364,11 @@ class ReactionSystem:
             Subindex of reaction to calculate reactant flux.    
         
         """
-        material_array = as_material_array(material,
-                                           self._basis,
-                                           self._phases,
-                                           self._chemicals)
-        values = get_array_values(material_array)
-        preconverted_material = values.copy()
+        material_array, config = as_material_array(
+            material.copy(), self._basis, self._phases, self._chemicals
+        )
+        isproperty = isinstance(material_array, property_array)
+        preconverted_material = material_array.value if isproperty else material_array
         reactions = self.reactions
         for i, rxn in enumerate(reactions):
             if i == index: break
