@@ -80,4 +80,4 @@ def __repr__(self):
     return f"<{type(self).__name__}: {self.ID}>"
 
 def __str__(self):
-    return self.ID
+    return self.ID or "."
